package mikroe.com.blepclick;

import android.app.Activity;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.ToggleButton;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import mikroe.com.blepclick.BleLib.BluetoothLeService;
import mikroe.com.blepclick.BleLib.BtManager;


public class BlepControl extends Activity {
    private final static String TAG = BlepControl.class.getSimpleName();

    ////////////////////////////////////////////////////////////////////////////////////////////////
    //                                             BLE                                            //
    ////////////////////////////////////////////////////////////////////////////////////////////////
    public static final String EXTRAS_DEVICE_NAME = "DEVICE_NAME";
    public static final String EXTRAS_DEVICE_ADDRESS = "DEVICE_ADDRESS";

    private BluetoothLeService mBluetoothLeService;
    private BluetoothGattService mBluetoothGattService;
    private BtManager mBtManager;

    private boolean mConnected = false;

    private String mDeviceName;
    private String mDeviceAddress;


    BluetoothGattCharacteristic blepTx = null;
    BluetoothGattCharacteristic blepRx = null;

    public static BluetoothDevice blepDevice = null;

    private TimerTask mTimerTask;
    Timer boardResponseTimer = new Timer();
    final Handler handler = new Handler();


    private ToggleButton tbPD0;
    private ToggleButton tbPD1;
    private ToggleButton tbPD2;
    private ToggleButton tbPD3;
    private ImageView ledPD4;
    private ImageView ledPD5;
    private ImageView ledPD6;
    private ImageView ledPD7;

    private TextView txtAdcValue;
    private ProgressBar pbAdcValue;

    private static final int[] led_state = new int[] {
            R.drawable.leds_icon_off,
            R.drawable.leds_icon_on,
    };


    private byte txBuffer[] = new byte[]{0x03,0,0};

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_blep_control);


        final Intent intent = getIntent();

        mDeviceName = intent.getStringExtra(EXTRAS_DEVICE_NAME);
        mDeviceAddress = intent.getStringExtra(EXTRAS_DEVICE_ADDRESS);


        getActionBar().setTitle(mDeviceName);
        getActionBar().setDisplayHomeAsUpEnabled(true);
        mBtManager = BtManager.getInstance();
        mBtManager.startBluetoothLeService(this, mServiceConnection);

        tbPD0 = (ToggleButton) findViewById(R.id.togglePD0);
        tbPD1 = (ToggleButton) findViewById(R.id.togglePD1);
        tbPD2 = (ToggleButton) findViewById(R.id.togglePD2);
        tbPD3 = (ToggleButton) findViewById(R.id.togglePD3);

        ledPD4 = (ImageView) findViewById(R.id.swPD4);
        ledPD5 = (ImageView) findViewById(R.id.swPD5);
        ledPD6 = (ImageView) findViewById(R.id.swPD6);
        ledPD7 = (ImageView) findViewById(R.id.swPD7);

        txtAdcValue = (TextView) findViewById(R.id.adcValue);
        pbAdcValue = (ProgressBar) findViewById(R.id.progressBar);



    }


    @Override
    protected void onResume() {
        super.onResume();

        registerReceiver(mGattUpdateReceiver, makeGattUpdateIntentFilter());
        if (mBluetoothLeService != null) {
            final boolean result = mBluetoothLeService.connect(mDeviceAddress);

        }

    }

    @Override
    protected void onPause() {
        super.onPause();
        //unregisterReceiver(mGattUpdateReceiver);
    }

    @Override
    protected void onDestroy() {

        super.onDestroy();
        stopTimerTask();
        mConnected = false;
        mBluetoothLeService.disconnect(mDeviceAddress);
        unbindService(mServiceConnection);
        mBluetoothLeService = null;

    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.blep_control, menu);
        if (mConnected) {
            menu.findItem(R.id.menu_connect).setVisible(false);
            menu.findItem(R.id.menu_disconnect).setVisible(true);
        } else {
            menu.findItem(R.id.menu_connect).setVisible(true);
            menu.findItem(R.id.menu_disconnect).setVisible(false);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        switch(item.getItemId()) {
            case R.id.menu_connect:
                mBluetoothLeService.connect(mDeviceAddress);
                return true;
            case R.id.menu_disconnect:
                mBluetoothLeService.disconnect(mDeviceAddress);
                return true;
            case android.R.id.home:
                onBackPressed();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }



    private void updateConnectionState(final int resourceId) {
       //ToDo:
        if(mConnected) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    doTimerTask();
                }
            });
        }

    }


    private void clearUI() {
        //toDo
    }



    private void displayData(int SwitchState, int AdcVal) {

        if ((SwitchState & 0x10) == 0){
            ledPD4.setImageResource(led_state[0]);
        }else{
            ledPD4.setImageResource(led_state[1]);
        }

        if ((SwitchState & 0x20) == 0){
            ledPD5.setImageResource(led_state[0]);
        }else{
            ledPD5.setImageResource(led_state[1]);
        }

        if ((SwitchState & 0x40) == 0){
            ledPD6.setImageResource(led_state[0]);
        }else{
            ledPD6.setImageResource(led_state[1]);
        }

        if ((SwitchState & 0x80) == 0){
            ledPD7.setImageResource(led_state[0]);
        }else{
            ledPD7.setImageResource(led_state[1]);
        }

        //ADC

        txtAdcValue.setText(String.format("%04d",AdcVal));

        pbAdcValue.setProgress(AdcVal);

    }


    private void makeChange() {
        byte tState = 0;
        if(mConnected) {
            if(tbPD0.isChecked()){tState += 0x01;}
            if(tbPD1.isChecked()){tState += 0x02;}
            if(tbPD2.isChecked()){tState += 0x04;}
            if(tbPD3.isChecked()){tState += 0x08;}

            txBuffer[0] = 0x03;                                                                //Data length
            txBuffer[1] = 0x50;                                                                //Cmd get data
            txBuffer[2] = tState;                                                              //Output port state
            if(blepTx != null) {
                blepTx.setValue(txBuffer);
                mBluetoothLeService.writeCharacteristic(blepTx);                              //Send data
            }


        }
    }


    //Check ADC
    public void doTimerTask(){
        mTimerTask = new TimerTask() {
            public void run() {
                makeChange();
            }};
        // public void schedule (TimerTask task, long delay, long period)
        boardResponseTimer.schedule(mTimerTask, 100, 200);  //
    }

    public void stopTimerTask(){
        if(mTimerTask!=null){
            handler.post(new Runnable() {
                public void run() {
                    clearUI();
                }
            });
            mTimerTask.cancel();
            mTimerTask=null;
        }
    }


    ////////////////////////////////////////////////////////////////////////////////////////////////
    //                                          BLE SERVICE                                       //
    ////////////////////////////////////////////////////////////////////////////////////////////////


    // Code to manage Service lifecycle.
    private final ServiceConnection mServiceConnection = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName componentName, IBinder service) {
            mBluetoothLeService = ((BluetoothLeService.LocalBinder) service).getService();
            if (!mBluetoothLeService.initialize()) {

                finish();
            }
            // Automatically connects to the device upon successful start-up initialization.
            mBluetoothLeService.connect(mDeviceAddress);
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {
            mBluetoothLeService = null;
        }
    };

    // Handles various events fired by the Service.
    // ACTION_GATT_CONNECTED: connected to a GATT server.
    // ACTION_GATT_DISCONNECTED: disconnected from a GATT server.
    // ACTION_GATT_SERVICES_DISCOVERED: discovered GATT services.
    // ACTION_DATA_AVAILABLE: received data from the device.  This can be a result of read
    //                        or notification operations.
    private final BroadcastReceiver mGattUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            final String action = intent.getAction();
            int status = intent.getIntExtra(BluetoothLeService.EXTRA_STATUS, BluetoothGatt.GATT_SUCCESS);

            if (BluetoothLeService.ACTION_GATT_CONNECTED.equals(action)) {
                mConnected = true;
                updateConnectionState(R.string.connected);
                invalidateOptionsMenu();
                if(BluetoothLeService.getBtGatt() != null)
                {
                    BluetoothLeService.getBtGatt().discoverServices();
                }
            } else if (BluetoothLeService.ACTION_GATT_DISCONNECTED.equals(action)) {
                mConnected = false;
                updateConnectionState(R.string.disconnected);
                invalidateOptionsMenu();
                clearUI();
            } else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED.equals(action)) {

                GetBlepGattService(mBluetoothLeService.getSupportedGattServices());

                if (mBluetoothGattService != null) {


                    mBluetoothLeService.setCharacteristicNotification(blepRx,true);

                }

            }
            else if (BluetoothLeService.ACTION_DATA_NOTIFY.equals(action)) {
                // Notification
                byte  [] value = intent.getByteArrayExtra(BluetoothLeService.EXTRA_DATA);
                String uuidStr = intent.getStringExtra(BluetoothLeService.EXTRA_UUID);
                onCharacteristicChanged(uuidStr, value);
            } else if (BluetoothLeService.ACTION_DATA_WRITE.equals(action)) {
                // Data written
                String uuidStr = intent.getStringExtra(BluetoothLeService.EXTRA_UUID);
                onCharacteristicWrite(uuidStr,status);
            } else if (BluetoothLeService.ACTION_DATA_READ.equals(action)) {
                // Data read
                String uuidStr = intent.getStringExtra(BluetoothLeService.EXTRA_UUID);
                byte  [] value = intent.getByteArrayExtra(BluetoothLeService.EXTRA_DATA);
                onCharacteristicsRead(uuidStr,value,status);
            }
        }
    };


    //List services and find our pref. service
    private void GetBlepGattService(List<BluetoothGattService> gattServices) {
        if (gattServices == null) return;
        String uuid = null;
        mBluetoothGattService = null;
        // Loops through available GATT Services.
        for (BluetoothGattService gattService : gattServices) {
            uuid = gattService.getUuid().toString();
            //Log.d(TAG, uuid);
            if(uuid.equals(BlepUtils.BLEP_SERVICE) ) {

                //Log.d(TAG, "Find BLEP SERVICE");
                mBluetoothGattService = gattService; //Set our prefered service
                // Loops through available Characteristics.
                List<BluetoothGattCharacteristic> gatCharacteristics =
                        gattService.getCharacteristics();
                for (BluetoothGattCharacteristic gattCharacteristic : gatCharacteristics) {

                    uuid = gattCharacteristic.getUuid().toString();

                    if(uuid.equals(BlepUtils.nRF_TX)){

                        blepTx = gattCharacteristic;
                        //Log.d(TAG, "Find TX Charasteristics");
                    }

                    if(uuid.equals(BlepUtils.nRF_RX)){

                        blepRx = gattCharacteristic;
                        //Log.v(TAG, "Find RX Charasteristics");
                    }

                }
            }
        }

    }


    ////////////////////////////////////////////////////////////////////////////////////////////////
    //                                        EVENTS                                              //
    ////////////////////////////////////////////////////////////////////////////////////////////////

    private static IntentFilter makeGattUpdateIntentFilter() {
        final IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothLeService.ACTION_GATT_CONNECTED);
        intentFilter.addAction(BluetoothLeService.ACTION_GATT_DISCONNECTED);
        intentFilter.addAction(BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED);
        intentFilter.addAction(BluetoothLeService.ACTION_DATA_NOTIFY);
        intentFilter.addAction(BluetoothLeService.ACTION_DATA_READ);
        intentFilter.addAction(BluetoothLeService.ACTION_DATA_WRITE);
        return intentFilter;
    }

    private void onCharacteristicWrite(String uuidStr, int status) {

        //ToDo
        //User code
    }

    private void onCharacteristicChanged(String uuidStr, byte[] value) {

        if (value != null && value.length > 0) {
            int pstate = BlepUtils.getIntValue(value,BlepUtils._UINT8,0);
            int adc = BlepUtils.getIntValue(value,BlepUtils._UINT16,2);
            displayData(pstate,adc);

        }
    }

    private void onCharacteristicsRead(String uuidStr, byte[] value, int status) {

        if (value != null && value.length > 0) {
            final StringBuilder stringBuilder = new StringBuilder(value.length);
            for (byte byteChar : value) {
                stringBuilder.append(String.format("%02X ", byteChar));
            }

        }
    }
}